@@ -60,17 +60,35 @@ read.dta <- function(file, convert.dates = TRUE,
             warning("'missing.type' only applicable to version >= 8 files")
     }
 
+    convert_dt_c <- function(x)
+        as.POSIXct((x+0.1)/1000, origin = "1960-01-01") # avoid rounding down
+
+    convert_dt_C <- function(x) {
+        ls <- .leap.seconds + seq_along(.leap.seconds)
+        z <- (x+0.1)/1000 # avoid rounding down
+        z <- z - rowSums(outer(z, ls, ">="))
+        as.POSIXct(z, origin = "1960-01-01")
+    }
+
     if (convert.dates) {
         ff <- attr(rval, "formats")
         ## dates <- grep("%-*d", ff)
         ## Stata 12 introduced 'business dates'
         ## 'Formats beginning with %t or %-t are Stata's date and time formats.'
         ## but it seems some are earlier.
-        dates <- if (attr(rval, "version") >= 8L) grep('%-*d|^%t|%-t', ff)
+        ## The dta_115 description suggests this is too inclusive:
+        ## 'Stata has an old *%d* format notation and some datasets
+        ##  still have them. Format *%d*... is equivalent to modern
+        ##  format *%td*... and *%-d*... is equivalent to *%-td*...'
+
+        dates <- if (attr(rval, "version") >= 8L) grep('^%(-|)(d|td)', ff)
         else grep("%-*d", ff)
         ## avoid as.Date in case strptime is messed up
-        base <- structure(-3653, class = "Date")
-        for(v in dates) rval[[v]] <- base+rval[[v]]
+        base <- structure(-3653L, class = "Date") # Stata dates are integer vars
+        for(v in dates) rval[[v]] <- structure(base + rval[[v]], class = "Date")
+
+        for(v in grep("%tc", ff)) rval[[v]] <- convert_dt_c(rval[[v]])
+        for(v in grep("%tC", ff)) rval[[v]] <- convert_dt_C(rval[[v]])
     }
     if (convert.factors %in% c(TRUE, NA)) {
         if (attr(rval, "version") == 5L)
@@ -148,6 +166,9 @@ write.dta <-
         for(v in dates)
             dataframe[[v]] <- as.vector(round(julian(dataframe[[v]],
                                                      ISOdate(1960,1,1, tz=tz))))
+        ## It would be possible to write these as %tc format,
+        ## milliseconds since 01jan1960 00:00:00.000
+        ## dataframe[[v]] <- 1000*as.vector(as.POSIXct(dataframe[[v]], tz=tz) + 315619200)
     }
     convert.factors <- match.arg(convert.factors)
     factors <- which(sapply(dataframe,is.factor))
