@@ -36,4 +36,11 @@ write.dta(bq, "bq.dta", version = 12)
 str(read.dta('bq.dta'))
 unlink("bq.dta")
 
+## PR#15290
+bq <- read.dta("OneVarTwoValLabels.dta")
+str(bq)
+
+## Dates and date-times in Stata12
+read.dta("xxx12.dta")
+
 q()
